; ***********************************************************
pro extremes, dco, av, var
; ***********************************************************
; Procedure to get the statistical properties of the fields
; Input: dco=Field to analyze 
; Output: av = average
;         var = variance

; Initialize
sumd=0.0d0
sumqd=0.0d0   
dmin=1d36
dmax=-1d36

; Scan the fields
for i=0,n_elements(dco)-1 do begin
  sumd=sumd+dco[i]
  sumqd=sumqd+dco[i]^2
  dmin=min([dmin,dco[i]])
  dmax=max([dmax,dco[i]])
endfor

; Print out the results

wgt=real(nx*ny)
av=sumd/wgt
var=sqrt(sumqd/wgt-(sumd/wgt)^2)

print, ' Average, standard deviation, minimum, and maximum:'
print,av,var,dmin,dmax,	format="(1P,4G11.3)"

return
end

; ***********************************************************
pro fwdfourier, reafeld,cplxfeld
; ***********************************************************
; Forward Fourier transform with array resorting
; Input: reafeld = real input field
;	nx, ny = field dimensions
; Output: cplxfeld = complex transformed field

a=size(reafeld)
nx=a[1]
ny=a[2]

data=dcomplexarr(nx,ny)
cplxfeld=dcomplexarr(nx,ny)

; Put into the linear field for transformation, resort
xdiv=(nx+1)/2
ydiv=(ny+1)/2
data[0:xdiv-1,0:ydiv-1]=reafeld[nx-xdiv:nx-1,ny-ydiv:ny-1]
data[xdiv:nx-1,0:ydiv-1]=reafeld[0:nx-xdiv-1,ny-ydiv:ny-1]
data[0:xdiv-1,ydiv:ny-1]=reafeld[nx-xdiv:nx-1,0:ny-ydiv-1]
data[xdiv:nx-1,ydiv:ny-1]=reafeld[0:nx-xdiv-1,0:ny-ydiv-1]

; call real FFT
data=fft(data,1,/overwrite)

; Field, resort to put zero frequencies into the center
xdiv=nx/2
ydiv=ny/2
cplxfeld[0:xdiv-1,0:ydiv-1]=data[nx-xdiv:nx-1,ny-ydiv:ny-1]
cplxfeld[xdiv:nx-1,0:ydiv-1]=data[0:nx-xdiv-1,ny-ydiv:ny-1]
cplxfeld[0:xdiv-1,ydiv:ny-1]=data[nx-xdiv:nx-1,0:ny-ydiv-1]
cplxfeld[xdiv:nx-1,ydiv:ny-1]=data[0:nx-xdiv-1,0:ny-ydiv-1]

return
end

; **********************************************************
pro bckfourier, cplxfeld,reafeld
; ***********************************************************
; Backward Fourier transform with array resorting

a=size(cplxfeld)
nx=a[1]
ny=a[2]

data=dcomplexarr(nx,ny)
reafeld=dblarr(nx,ny)

; Put into the linear field for transformation, resort
xdiv=(nx+1)/2
ydiv=(ny+1)/2
data[0:xdiv-1,0:ydiv-1]=cplxfeld[nx-xdiv:nx-1,ny-ydiv:ny-1]
data[xdiv:nx-1,0:ydiv-1]=cplxfeld[0:nx-xdiv-1,ny-ydiv:ny-1]
data[0:xdiv-1,ydiv:ny-1]=cplxfeld[nx-xdiv:nx-1,0:ny-ydiv-1]
data[xdiv:nx-1,ydiv:ny-1]=cplxfeld[0:nx-xdiv-1,0:ny-ydiv-1]

; FFT
data=fft(data,-1,/overwrite)

; Fill the real field, resort, remove the factor from four
xdiv=nx/2
ydiv=ny/2
fac=double(1.0/(nx*ny))
reafeld[0:xdiv-1,0:ydiv-1]=data[nx-xdiv:nx-1,ny-ydiv:ny-1]*fac
reafeld[xdiv:nx-1,0:ydiv-1]=data[0:nx-xdiv-1,ny-ydiv:ny-1]*fac
reafeld[0:xdiv-1,ydiv:ny-1]=data[nx-xdiv:nx-1,0:ny-ydiv-1]*fac
reafeld[xdiv:nx-1,ydiv:ny-1]=data[0:nx-xdiv-1,0:ny-ydiv-1]*fac

return
end
