; **************************************************************
pro extractdeltaparms, widget, drad=drad, signif=signif, filter=filter, $ 
	continue=continue, periodic=periodic, datawidget=datawidget, $
	wgtwidget=wgtwidget, fitwidget=fitwidget
; **************************************************************

WIDGET_CONTROL, widget, GET_UVALUE=info
WIDGET_CONTROL, info.drad, GET_VALUE=dx
IF strnumber(dx[0],drad) eq 0 THEN drad=0.0d0

WIDGET_CONTROL, info.filter, GET_VALUE=nvec
filter=nvec[0]

WIDGET_CONTROL, info.signif, GET_VALUE=sg
IF strnumber(sg[0],signif) eq 0 THEN signif=0.0d0

; WIDGET_CONTROL, info.errmet, GET_VALUE=nvec
; benscherr=nvec[0]

WIDGET_CONTROL, info.cont, GET_VALUE=nvec
if nvec[0] eq 1 then continue=1 else continue=0
if nvec[0] eq 0 then periodic=1 else periodic=0

widgets=info.widgets
datawidget=widgets.data
wgtwidget=widgets.wgt
fitwidget=widgets.fit

return
end

; **************************************************************
pro writedeltaparms, widget, datawidget=datawidget, $
	wgtwidget=wgtwidget, fitwidget=fitwidget
; **************************************************************

WIDGET_CONTROL, widget, GET_UVALUE=info
widgets=info.widgets

if n_elements(datawidget) ne 0 then widgets.data=datawidget
if n_elements(wgtwidget) ne 0 then widgets.wgt=wgtwidget
if n_elements(fitwidget) ne 0 then widgets.fit=fitwidget
info.widgets=widgets
WIDGET_CONTROL, widget, SET_UVALUE=info

return
end

; **************************************************************
pro extractwidgetdata, widget, data=fdata, weights=wgt, pixel=pixel, $
     blank=blank, titles=titles, delta=delta, limits=limits, $
     beam=beam, fit=fit, filenames=filenames
; **************************************************************

WIDGET_CONTROL, widget, GET_UVALUE=info
WIDGET_CONTROL, info.parent, GET_UVALUE=dwid
WIDGET_CONTROL, dwid.data, GET_UVALUE=datastruct

fdata=datastruct.data
wgt=datastruct.weights
delta=datastruct.delta

pixel=datastruct.pixel
blank=datastruct.blank
titles=datastruct.titles

limits=datastruct.limits
beam=datastruct.beam
fit=datastruct.fit
filenames=datastruct.filenames

return
end

; **************************************************************
pro writewidgetdata, widget, data=fdata, weights=wgt, pixel=pixel, $
     blank=blank, title=titles, delta=delta, limits=limits, $
     beam=beam, fit=fit, filenames=filenames
; **************************************************************

WIDGET_CONTROL, widget, GET_UVALUE=info
WIDGET_CONTROL, info.parent, GET_UVALUE=dwid
WIDGET_CONTROL, dwid.data, GET_UVALUE=datastruct

if n_elements(pixel) ne 0 then datastruct.pixel=pixel
if n_elements(blank) ne 0 then datastruct.blank=blank
if n_elements(titles) ne 0 then datastruct.titles=titles
if n_elements(limits) ne 0 then datastruct.limits=limits
if n_elements(beam) ne 0 then datastruct.beam=beam
if n_elements(filenames) ne 0 then datastruct.filenames=filenames

; Special treatment for fields as size variation is not
; allowed within a structure
; Data field
if n_elements(fdata) ne 0 then begin
if total(size(datastruct.data) ne size(fdata)) then begin
        datastruct = {data:fdata,  $
	weights:datastruct.weights, pixel:datastruct.pixel, $
	blank:datastruct.blank, titles:datastruct.titles, $
	delta:datastruct.delta, limits:datastruct.limits, $
	beam:datastruct.beam, fit:datastruct.fit, $
	filenames:datastruct.filenames}
endif else begin
		datastruct.data=fdata
	endelse
endif
; Weight field
if n_elements(wgt) ne 0 then begin
if total(size(datastruct.weights) ne size(wgt)) then begin
        datastruct = {data:datastruct.data,  $
	weights:wgt, pixel:datastruct.pixel, $
	blank:datastruct.blank, titles:datastruct.titles, $
	delta:datastruct.delta, limits:datastruct.limits, $
	beam:datastruct.beam, fit:datastruct.fit, $
	filenames:datastruct.filenames}
endif else begin
		datastruct.weights=wgt
	endelse
endif
; Delta field
if n_elements(delta) ne 0 then begin
if total(size(datastruct.delta) ne size(delta)) then begin
        datastruct = {data:datastruct.data,  $
	weights:datastruct.weights, pixel:datastruct.pixel, $
	blank:datastruct.blank, titles:datastruct.titles, $
	delta:delta, limits:datastruct.limits, $
	beam:datastruct.beam, fit:datastruct.fit, $
	filenames:datastruct.filenames}
endif else begin
		datastruct.delta=delta
	endelse
endif
; Fit field
if n_elements(fit) ne 0 then begin
if total(size(datastruct.fit) ne size(fit)) then begin
        datastruct = {data:datastruct.data,  $
	weights:datastruct.weights, pixel:datastruct.pixel, $
	blank:datastruct.blank, titles:datastruct.titles, $
	delta:datastruct.delta, limits:datastruct.limits, $
	beam:datastruct.beam, fit:fit, $
	filenames:datastruct.filenames}
endif else begin
		datastruct.fit=fit
	endelse
endif
WIDGET_CONTROL, dwid.data, SET_UVALUE=datastruct

return
end

; *******************************************************************
pro plotgraph, res, titles=titles
; *******************************************************************
; Plot the Delta variance as in the normal computation

if n_elements(titles) ne 2 then titles=["lag","!4r!DD!3!U2!N"]

plot, res[*,0], res[*,1], psym=3, /xlog, /ylog, $
  xtitle=titles[0], ytitle=titles[1]
oplot, res[*,0], res[*,1], psym=2, color=visiblecolor(0.25)
errplot, res[*,0], res[*,1]-res[*,2], res[*,1]+res[*,2]
return
end

; **************************************************************
function deltavariancechecks, wid, feld, wgt, pixel=pixel,drad=drad
; **************************************************************
; Subroutine to check inputs

result=0
if comparefieldsizes(feld,wgt) ne 0 then begin
  bummer, wid,['Size of the data field does not match', $
		'the size of the weights field!','',$
		'Delta variance calculation aborted.','']
  return, 1
endif
if (max(feld)-min(feld) eq 0) then begin
  bummer, wid,['No valid input data field defined!','',$
		'Delta variance calculation aborted.','']
  return, 1
endif
if n_elements(pixel) gt 0 then begin
	if min(pixel gt 0.0) eq 0 then begin
	  bummer, wid,['Pixel size must be positive!','',$
		'Delta variance calculation aborted.','']
	  return, 1
	endif
endif
if n_elements(drad) gt 0 then begin
	if drad le 1.0 then begin
	  bummer, wid,['Diameter ratio must be greater then 1 !','',$
		'Delta variance calculation aborted.','']
	  return, 1
	endif
endif
return, result
end

; **************************************************************
PRO okhandler, event
; **************************************************************
; Event handler for message widget - only ok allowed

IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_BUTTON') THEN BEGIN
 WIDGET_CONTROL, event.top, /DESTROY
ENDIF

RETURN
END

; **************************************************************
pro bummer, parent, texte
; **************************************************************
; Error message widget showing the text messages

boldfont="-misc-fixed-bold-r-normal--13-120-75-75-c-70-iso8859-1"

nlin=n_elements(texte)
win=widget_base(group_leader=parent,column=1,/base_align_center, $
	/modal, title="Error",event_pro="okhandler")
for i=0,nlin-1 do begin
  message=widget_label(win,font=boldfont,value=texte[i])
endfor
okbutton = WIDGET_BUTTON(win, VALUE='Ok')

widget_control, win, default_button=okbutton
widget_control, win, /realize
return
end

; **************************************************************
pro deltavarcompute, parent
; **************************************************************
; Routine called for computing the Delta variance in widget

extractdeltaparms, parent, drad=drad, filter=filter, $
			periodic=periodic, continue=continue, $
			signif=signif, fitwidget=fwd
extractwidgetdata, parent, data=feld, pixel=pixel, weights=wgt, $
			beam=beam
if deltavariancechecks(parent,feld,wgt,pixel=pixel,drad=drad) eq 0 then begin
	widget_control, /hourglass
	res=deltavariance(feld,(wgt > 0.0d0),pixel=pixel,drad=drad, $
		periodic=periodic, filter=filter, continue=continue, $
		signifexp=signif)
	widget_control, parent, /clear_events
	; Predefine limits
	last=n_elements(res[*,0])
	fitrange=[max([beam,res[0,0]*1.1d0]),res[last-1,0]*0.9d0]
	writewidgetdata, parent, delta=res, limits=fitrange
	; Open window
	exists=widget_info(fwd,/valid_id)
	if exists then widget_control, fwd, /destroy
	fwd=fitwidget(parent)
  	writedeltaparms, parent, fitwidget=fwd
endif

return
end

; **************************************************************
PRO DeltaVarEvent, event
; **************************************************************
; Event handler for the fit widget of the Delta variance

IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_BUTTON') THEN BEGIN
WIDGET_CONTROL, event.id, GET_VALUE=button_value

; Button handling
CASE button_value OF
; Help buttons
'About': XDISPLAYFILE, "about.txt", title="About", height=7, width=25
'Delta variance help': XDISPLAYFILE, "deltahelp.txt", $
			title="Help on Delta variance computation"
'Exit': WIDGET_CONTROL, event.top, /DESTROY
; Window buttons
'Data field': begin
		extractdeltaparms, event.top, datawidget=fwd
		exists=widget_info(fwd,/valid_id)
		if not exists then begin
			fwd=feldwidget(event.top)
			writedeltaparms, event.top, datawidget=fwd
		endif else widget_control, fwd, /show
	      end
'Weight field': begin
		extractdeltaparms, event.top, wgtwidget=fwd
		exists=widget_info(fwd,/valid_id)
		if not exists then begin
			fwd=wgtwidget(event.top)
			writedeltaparms,event.top,  wgtwidget=fwd
		endif else widget_control, fwd, /show
	        end 
'Delta variance': begin
		extractdeltaparms, event.top, fitwidget=fwd
		exists=widget_info(fwd,/valid_id)
		if not exists then begin
			fwd=fitwidget(event.top)
			writedeltaparms, event.top, fitwidget=fwd
		endif else widget_control, fwd, /show
	        end
; File buttons
'Read data field': begin
		if loadfield(event.top) eq 0 then begin
		  extractdeltaparms, event.top, datawidget=fwd, wgtwidget=fwt
		  exists=widget_info(fwd,/valid_id)
		  if exists then widget_control, fwd, /destroy
	  	  fwd=feldwidget(event.top)
		  writedeltaparms, event.top, datawidget=fwd
		  ; Create simple weights field
		  extractwidgetdata, event.top, data=feld, blank=blank
		  wgt=simpleweights(feld,blank=blank)
		  writewidgetdata, event.top, weights=wgt
		  exists=widget_info(fwt,/valid_id)
		  if exists then begin
		    widget_control, fwt, /destroy
	  	    fwt=wgtwidget(event.top)
		    writedeltaparms, event.top, wgtwidget=fwt
		  endif
		endif
		   end
'Read weights': begin
		if loadfield(event.top,/weights) eq 0 then begin
		  extractdeltaparms, event.top, wgtwidget=fwd
		  exists=widget_info(fwd,/valid_id)
		  if exists then widget_control, fwd, /destroy
	  	  fwd=wgtwidget(event.top)
		  writedeltaparms, event.top, wgtwidget=fwd
		endif
		end
'Write Delta variance': dummy=writedeltavar(event.top)
; Compute buttons
'Compute weights': BEGIN
		extractwidgetdata, event.top, data=feld, blank=blank
		wgt=simpleweights(feld,blank=blank)
		writewidgetdata, event.top, weights=wgt
		; Open window
		extractdeltaparms, event.top, wgtwidget=fwd
		exists=widget_info(fwd,/valid_id)
		if exists then widget_control, fwd, /destroy
	  	fwd=wgtwidget(event.top)
		writedeltaparms, event.top, wgtwidget=fwd
		   END
'Compute Delta variance': deltavarcompute, event.top
ENDCASE
ENDIF

; Enter on input fields is equal to Update above
IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_TEXT_CH') THEN BEGIN
 if event.type eq 0 then deltavarcompute, event.top
ENDIF

RETURN
END
