; *******************************************************************
function findblanked, feld, blank, count=count
; *******************************************************************
; Find pixels which are blanked

span=(1.0d-3*abs(blank)) > (1.0d-5*stddev(feld,/nan))
blankindices=where((abs(feld-blank) lt span) or (~ finite(feld)),count)
return, blankindices
end

; *******************************************************************
function simpleweights, feld, blank=blank
; *******************************************************************
; Compute the weighting field from blanks without independent 
; weighing information

; Field size
a=size(feld)

; Numerical control parameters
if n_elements(blank) ne 1 then blank=-1.0d30

; Creat field of weights
w=make_array(size=a,value=1.0d0)
blankindices=findblanked(feld, blank, count=count)
if count gt 0 then w[blankindices]=0.0d0

return, w
end

; **************************************************************
function comparefieldsizes, field1, field2
; **************************************************************
; Compare the size of two fields

size1=size(field1)
size2=size(field2)
len=n_elements(size1)
if n_elements(size2) ne len then return, 1

size1[len-2]=0l
size2[len-2]=0l
if total(size1 ne size2) eq 0 then return, 0 else return, 1
end

; *******************************************************************
function deltavariance, feld, wfeld, pixel=pixel, filter=filter, $
   drad=v, cut=cut, periodic=periodic, continue=continue, $
   signifexp=signifexp
; *******************************************************************
; Outer program to compute the Delta variance analysis 

; Field size
a=size(feld)
nx=a[1]
ny=a[2]

; Pixel size
if n_elements(pixel) eq 0 then pixel=[1.0d0,1.0d0] $
 else if n_elements(pixel) eq 1 then pixel=[pixel,pixel] 

; Numerical control parameters
if not keyword_set(v) then v=3.0d0
if not keyword_set(cut) then cut=1.0d0

; First version, already periodic
if keyword_set (continue) then begin
 dat=mirror(feld)
 weight=mirror(wfeld)
 effcut=cut*2.0d0
endif else if keyword_set (periodic) then begin
 dat=feld
 weight=wfeld
 effcut=cut
endif else begin
  ; Pad boundaries with zeros
  origsize=min([nx,ny])
  blankwidth=fix(origsize*(v+1.0d0)/(2.0d0*cut*v))+1
  dat=zeropad(feld, blankwidth)
  weight=zeropad(wfeld, blankwidth)
  newsize=min([nx+blankwidth[0]+blankwidth[1],$
	ny+blankwidth[2]+blankwidth[3]],subscript)
  effcut=double(newsize)/a[subscript+1]*cut
endelse

dp=delta_weighted(dat,weight, pixel=pixel, drad=v, cut=effcut, $
			filter=filter,signifexp=signifexp)
if keyword_set (continue) then dp[*,2]=dp[*,2]*2

return, dp
end
