; ******************************************************************
PRO DeltaVarWidget, feld=feld, wfeld=wfeld, blank=blank, pixel=pixel, $
	titles=titles, delta=delta, fitrange=fitrange, beam=beam, drad=drad
; ******************************************************************
; Main program - Definition of the Widget for the Delta variance

; Compile everything
resolve_routine, "bckfourier"
resolve_routine, "deltavariance", /is_function
resolve_routine, "fitfunction", /is_function
resolve_routine, "feldwidget", /is_function
resolve_routine, "wgtwidget", /is_function
resolve_routine, "fitwidget", /is_function
resolve_routine, "deltavarevent"

; ----------------------------------
; First part - data handling
; ----------------------------------
; Data field
; ----------------------------------
; Check for the format of the input field
rs=size(res)
if (rs[0] ne 2) then begin
	feld=make_array(100,100,value=1.0d0)
endif

; Check for special titles
if n_elements(titles) ne 2 then titles=['!4D!3x','!4D!3y','']

; Blanking
if n_elements(blank) ne 1 then blank=-1.0d30

; Pixel size
if n_elements(pixel) eq 0 then pixel=[1.0d0,1.0d0] $
   else if n_elements(pixel) eq 1 then pixel=[pixel,pixel] 

; ----------------------------------
; Weight field
; ----------------------------------
; Check for the format of the input field
rs=size(wfeld)
if (rs[0] ne 2) then begin
	wfeld=make_array(size=size(feld),value=1.0d0)
endif

; ----------------------------------
; Delta variances
; ----------------------------------
; Check for the format of the input field
rs=size(delta)
if (rs[0] ne 2) or (rs[1] lt 3) or (rs[2] lt 2) or (rs[2] gt 4) then begin
  delta=make_array(50,4,value=0.0d0)
  rs=size(delta)
endif
; Create weights if not present, create  fit data if not present
case rs[2] of
2: delta=[[delta[*,0]],[delta[*,1]],[make_array(rs[1],value=0.0d0)],$
					[make_array(rs[1],value=0.0d0)]]
3: delta=[[delta[*,0]],[delta[*,1]],[delta[*,2]],[make_array(rs[1],value=0.0d0)]]
4:
endcase

; Limits
if n_elements(fitrange) eq 2 then xlimits=[min(fitrange),max(fitrange)] $
else xlimits=[min(delta[*,0]),max(delta[*,0])]

; Beam
if n_elements(beam) ne 1 then beam=0.0d0

; ---------------------------------------
; Extra data
; ---------------------------------------
; Diameter ratio
if n_elements(drad) ne 1 then drad=1.5d0
dradstring=STRTRIM(string(drad, format='(G9.3)'),2)
; Significance exponent
signifexp=1.0d0
signifstring=STRTRIM(string(signifexp, format='(G9.3)'),2)

fit=[[0.0d0,0.0d0,0.0d0],[0.0d0,0.0d0,0.0d0]]
filenames = {datafile:'', wgtfile:'', resfile:'', printfile:'hardcopy.ps', dir:'.'}

; ---------------------------------------
; Store
; ---------------------------------------
datastruct = {data:feld, weights:wfeld, pixel:pixel, blank:blank, titles:titles, $
	delta:delta, limits: xlimits, beam:beam, fit:fit, filenames:filenames}

; ----------------------------------
; Now create the widget
; ----------------------------------
boldfont="-misc-fixed-bold-r-normal--13-120-75-75-c-70-iso8859-1"

tlb = WIDGET_BASE(ROW=5, TITLE="Delta variance calculator",mbar=bar)

;
; Menu bar
;
file_menu = widget_button(bar, value="File", /menu)
feldread_btn = widget_button(file_menu, value="Read data field")
wgtread_btn = widget_button(file_menu, value="Read weights")
; Not implemented yet
; resread_btn = widget_button(file_menu, value="Read Delta variance")
reswrite_btn = widget_button(file_menu, value="Write Delta variance")
bye_button = WIDGET_BUTTON(file_menu, VALUE='Exit')

compute_menu = widget_button(bar, value="Compute", /menu)
deltacompute = widget_button(compute_menu, value="Compute weights")
weightcompute = widget_button(compute_menu, value="Compute Delta variance")

window_menu = widget_button(bar, value="Windows", /menu)
feldwindow_btn = widget_button(window_menu, value="Data field")
weightwindow_btn = widget_button(window_menu, value="Weight field")
fitwindow_btn = widget_button(window_menu, value="Delta variance")

help_menu = widget_button(bar, value="Help", /help)
about_btn = widget_button(help_menu, value="About")
help_btn = widget_button(help_menu, value="Delta variance help")

;
; Parameter area
;
filterwd = WIDGET_BASE(tlb, ROW=1)
filternames=["French hat","Mexican hat"]
filter= CW_BGROUP(filterwd, filternames, column=1, LABEL_LEFT='Filter type:   ', $
		SET_VALUE=1,/EXCLUSIVE)

diameterwd = WIDGET_BASE(tlb, ROW=1)
diameter_label = WIDGET_LABEL(diameterwd, VALUE='Diameter ratio: ')
dradfield = WIDGET_TEXT(diameterwd, VALUE=dradstring, /EDITABLE, XSIZE=8)

contwd = WIDGET_BASE(tlb, ROW=1)
contnames=["Map is periodic","Contine by mirroring","Cut at edges"]
cont = CW_BGROUP(contwd, contnames, column=1, LABEL_LEFT="Edge treatment:", $
		SET_VALUE=2,/EXCLUSIVE)

; errwd = WIDGET_BASE(tlb, ROW=1)
; errnames=["Poisson error","Bensch error"]
; errmet = CW_BGROUP(errwd, errnames, column=1, LABEL_LEFT='Error bars:    ', $
; 		SET_VALUE=0,/EXCLUSIVE)

errwd = WIDGET_BASE(tlb, ROW=1)
errlab = WIDGET_BASE(errwd, COLUMN=1)
err_label1 = WIDGET_LABEL(errlab, VALUE='Weight exponent')
err_label2 = WIDGET_LABEL(errlab, VALUE='for error bars:')
errfld = WIDGET_BASE(errwd, COLUMN=1, /ALIGN_BOTTOM)
signiffield = WIDGET_TEXT(errfld, VALUE=signifstring, /EDITABLE, XSIZE=8, YSIZE=1)

buttonfld = WIDGET_BASE(tlb, COLUMN=2, /BASE_ALIGN_RIGHT)
update_label = WIDGET_LABEL(buttonfld, VALUE='                ')
update_button = WIDGET_BUTTON(buttonfld, VALUE='Compute Delta variance')

WIDGET_CONTROL, tlb, /REALIZE

widgets={data:0l, wgt:0l, fit:0l}
tlbinfo={parent:tlb, data:update_button, filter:filter, cont:cont, $
	signif:signiffield, drad:dradfield, widgets:widgets}
WIDGET_CONTROL, tlb, SET_UVALUE=tlbinfo
WIDGET_CONTROL, update_button, SET_UVALUE=datastruct

XMANAGER, 'DeltaVarWidget', tlb, EVENT='DeltaVarEvent'
; XMANAGER, 'DeltaVarWidget', tlb, EVENT='DeltaVarEvent', /no_block
END
