; **************************************************************
pro extractfeldparms, widget, pixel=pixel, blank=blank
; **************************************************************

WIDGET_CONTROL, widget, GET_UVALUE=info

pixel=[1.0d0,1.0d0]
WIDGET_CONTROL, info.dx, GET_VALUE=dx
IF strnumber(dx[0],px) eq 0 THEN pixel[0]=1.0d0 else pixel[0]=px
WIDGET_CONTROL, info.dy, GET_VALUE=dy
IF strnumber(dy[0],px) eq 0 THEN pixel[1]=1.0d0 else pixel[1]=px

WIDGET_CONTROL, info.blank, GET_VALUE=bw
IF strnumber(bw[0],blank) eq 0 THEN blank=0.0d0
blank=double(blank)

return
end

; **************************************************************
pro writefeldparms, widget, pixel=pixel, blank=blank, $
	status=status, onlystatus=onlystatus
; **************************************************************

WIDGET_CONTROL, widget, GET_UVALUE=info
if n_elements(status) ne 0 then WIDGET_CONTROL, info.status, SET_VALUE=status

if not keyword_set(onlystatus) then begin
   if n_elements(blank) ne 0 then  begin
	blankstring=STRTRIM(string(blank, format='(G9.3)'),2)
	WIDGET_CONTROL, info.blank, SET_VALUE=blankstring
   endif
   if n_elements(pixel) eq 1 then begin
	dxstring=STRTRIM(string(pixel, format='(G9.3)'),2)
	WIDGET_CONTROL, info.dx, SET_VALUE=dxstring
	WIDGET_CONTROL, info.dy, SET_VALUE=dxstring
   endif else if n_elements(pixel) eq 2 then begin
	dxstring=STRTRIM(string(pixel[0], format='(G9.3)'),2)
	WIDGET_CONTROL, info.dx, SET_VALUE=dxstring
	dystring=STRTRIM(string(pixel[1], format='(G9.3)'),2)
	WIDGET_CONTROL, info.dy, SET_VALUE=dystring
   endif
endif
return
end

; *******************************************************************
function niceblanks, feld, blank
; *******************************************************************
; Set the blanked value to some value within the normal range

newfeld=feld
blankindices=findblanked(feld, blank, count=count)
if count gt 0 then begin
  if min(feld) eq feld[blankindices[0]] then newfeld[blankindices]=max(feld)
  newfeld[blankindices]=min(newfeld)
endif

return, newfeld
end

; *******************************************************************
pro showfeld, feld, pixel=pixel, titles=titles, showcontours=showcontours
; *******************************************************************
; Show the overlay

a=size(feld)
if a[0] ne 2 then begin
  print, "Field is not two-dimensional."
  return
endif

if n_elements(titles) lt 2 then titles=['!4D!3x','!4D!3y','']
if n_elements(pixel) eq 0 then pixel=[1.0d0,1.0d0] 

if keyword_set(showcontours) then  $
  if max(feld)-min(feld) le 1.0d-2*total(feld)/n_elements(feld) $
	then nlevels=0 $
	else nlevels=3 $
else nlevels=0

xx=findgen(a[1])*pixel[0]
yy=findgen(a[2])*pixel[1]

overlay, feld, xx, yy, nlevels=nlevels, titles=titles
return
end

; **************************************************************
function getfitsfile, filename,pixel=pixel,titles=titles,blank=blank
; **************************************************************
; Grab the data field and the parameters from the FITS file

feld=readfits(filename,hdr)
xscl=headerscl(hdr,yscale=yscl,blank=blank,minmax=minmax, $
		xtitle=xtit,ytitle=ytit)
pixel=[abs(xscl[1]-xscl[0]),abs(yscl[1]-yscl[0])]

; Blanking treatment
; Step 1 - find invalid data
invalid=where(finite(feld) ne 1,count)
if n_elements(minmax) gt 1 then begin
      invalid2=where((feld lt minmax[0]) or (feld gt minmax[1]),count2)
      if count2 gt 0 then begin
	 if count gt 0 then invalid=[invalid,invalid2] $
	   else invalid=invalid2
         count=count+count2
      endif
endif

if not finite(blank) then blank=-1
; Compute blank if no blanking specified or blanks are NaN-ed
if blank eq -1 then begin
 ; Try to find blanking value
 if n_elements(minmax) gt 1 then begin
   if minmax[0] gt 1.001d0*min(feld) then blank=min(feld)
   if minmax[1] lt 0.999d0*max(feld) then blank=max(feld)
 endif
 ; Invent blanking value
 if blank eq -1 then begin
   valid=where(finite(feld))
   vmin=min(feld[valid])
   iv=fix(alog10(abs(vmin)))
   if iv lt 3 then blank=-1000.0d0 else blank=-10.0d0^(iv+1)
 endif
endif

; Assign blanks
if count gt 0 then feld[invalid]=blank

if xtit eq ' ' then xtit='x'
if ytit eq ' ' then ytit='y'
titles=[xtit,ytit,'']

; Make float
return, double(reform(feld))
end

; **************************************************************
function loadfield, widget, data=data, weights=weights
; **************************************************************
; Load a field from a fits file and store the parameters

extractwidgetdata, widget, filenames=filenames
if not keyword_set(weights) then file=filenames.datafile $
  else file=filenames.wgtfile
dir=filenames.dir
filter="*.fits"
filename=dialog_pickfile(file=file,path=dir,/must_exist,$
			filter=filter,get_path=dir,/read)

if (filename ne '') then begin
  feld=getfitsfile(filename,pixel=pixel,titles=titles,blank=blank)
  filenames.dir=dir
  if not keyword_set(weights) then begin
	filenames.datafile=filename
	beam=pixel[0]*2.0d0
  	writewidgetdata, widget, data=feld, pixel=pixel, beam=beam, $
		blank=blank, filenames=filenames, title=titles
  endif else begin
  	filenames.wgtfile=filename
	; blank to zero
        invalid=findblanked(feld, blank, count=count)
        if count gt 0 then feld[invalid]=0.0d0
  	writewidgetdata, widget, weights=feld, filenames=filenames
  endelse
  return, 0
endif else return, 1
end

; **************************************************************
PRO DeltaFeldEvent, event
; **************************************************************
; Event handler for the fit widget of the Delta variance

; Switch to active drawing area
WIDGET_CONTROL, event.top, GET_UVALUE=info
wset, info.draw_wid

IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_BUTTON') THEN BEGIN
WIDGET_CONTROL, event.id, GET_VALUE=button_value

; Button handling
CASE button_value OF
'Data file help': XDISPLAYFILE, "feldhelp.txt", title="Help on data files"
'Close': WIDGET_CONTROL, event.top, /DESTROY
'Update': BEGIN
	extractfeldparms, event.top, pixel=pixel, blank=blank
	extractwidgetdata, event.top, data=fdata, titles=titles, blank=oldblank
	if blank ne oldblank then begin
          oldblanked=findblanked(fdata, oldblank, count=count)
	  if count gt 0 then fdata[oldblanked]=blank
	endif
	plotfeld=niceblanks(fdata,blank)
	showfeld, plotfeld, pixel=pixel, titles=titles
  	statline=STRCOMPRESS(string(min(plotfeld),max(plotfeld), $
		format='("f(min) =",G10.4," , f(max) =",G10.4)'))
	writewidgetdata, event.top, data=fdata, pixel=pixel, blank=blank
	writefeldparms, event.top, status=statline, /onlystatus
	 END
'Load': BEGIN
	if loadfield(event.top) eq 0 then begin
	  extractwidgetdata, event.top, data=feld, pixel=pixel, $
		blank=blank, title=titles
	  plotfeld=niceblanks(feld,blank)
	  showfeld, plotfeld, pixel=pixel, titles=titles
  	  statline=STRCOMPRESS(string(min(plotfeld),max(plotfeld), $
		format='("f(min) =",G10.4," , f(max) =",G10.4)'))
	  writefeldparms, event.top, blank=blank, pixel=pixel, status=statline
	endif
	     END
ENDCASE
ENDIF

; Enter on input fields is equal to Update above
IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_TEXT_CH') THEN BEGIN
 if event.type eq 0 then begin
	extractfeldparms, event.top, pixel=pixel, blank=blank
	extractwidgetdata, event.top, data=fdata, titles=titles
	plotfeld=niceblanks(fdata,blank)
	showfeld, plotfeld, pixel=pixel, titles=titles
  	statline=STRCOMPRESS(string(min(plotfeld),max(plotfeld), $
		format='("f(min) =",G10.4," , f(max) =",G10.4)'))
	writewidgetdata, event.top, pixel=pixel, blank=blank
	writefeldparms, event.top, status=statline, /onlystatus
  endif
ENDIF

; Mouse click in the drawing widget - Examine values
IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_DRAW') THEN BEGIN
	; Button pressed
	if event.type eq 0 then begin
	xyz=CONVERT_COORD(event.x,event.y,/DEVICE, /TO_DATA)
	extractwidgetdata, event.top, data=fdata, titles=titles,pixel=pixel
	 inx=size(fdata)
	 xcoord=fix(xyz[0]/pixel[0]+0.5d0)
	 ycoord=fix(xyz[1]/pixel[1]+0.5d0)
	if ((xcoord ge 0) and (xcoord lt inx[1]) and $
			(ycoord ge 0) and (ycoord lt inx[2])) then begin
	   z=fdata[xcoord,ycoord]
	   statline=STRCOMPRESS(titles[0]+string(xyz[0], format='(" = ",G10.4,", ")')+ $
			titles[1]+string(xyz[1], format='(" = ",G10.4,", ")')+ $
			string(z, format='("I = ",G10.4)'))
	endif else statline="Out of data area"
	writefeldparms, event.top, status=statline, /onlystatus
	endif
ENDIF
RETURN
END

; ******************************************************************
function FeldWidget, father
; ******************************************************************
;
; ----------------------------------
; First part - data handling
; ----------------------------------
extractwidgetdata, father, data=feld, pixel=pixel, $
         blank=blank, titles=titles
; Input field
rs=size(feld)
plotfeld=niceblanks(feld,blank)

if (rs[0] ne 2) then begin
	statline="Empty field"
endif else begin
  	statline=STRCOMPRESS(string(min(plotfeld),max(plotfeld), $
		format='("f(min) =",G10.4," , f(max) =",G10.4)'))
endelse
; Blanking
blankstring=STRTRIM(string(blank, format='(G9.3)'),2)
; Pixel size
dxstring=STRTRIM(string(pixel[0], format='(G9.3)'),2)
dystring=STRTRIM(string(pixel[1], format='(G9.3)'),2)

; ----------------------------------
; Now create the widget
; ----------------------------------
boldfont="-misc-fixed-bold-r-normal--13-120-75-75-c-70-iso8859-1"

tlb= WIDGET_BASE(ROW=3, TITLE="Data field",mbar=bar, $
		group_leader=father, event_pro="DeltaFeldEvent")

;
; Menu bar
;
file_menu = widget_button(bar, value="File", /menu)
read_button = WIDGET_BUTTON(file_menu, VALUE='Load')
bye_button = WIDGET_BUTTON(file_menu, VALUE='Close')

help_menu = widget_button(bar, value="Help", /help)
help_btn = widget_button(help_menu, value="Data file help")

;
; Draw area
;
draw = WIDGET_DRAW(tlb, XSIZE=860, YSIZE=760, COLORS=128, RETAIN=2, /BUTTON_EVENTS)
status = WIDGET_LABEL(tlb, VALUE="Initstring", /DYNAMIC_RESIZE, FONT=boldfont) 

;
; Parameter area
;
commands = WIDGET_BASE(tlb, ROW=1)
blank_label = WIDGET_LABEL(commands, VALUE='Blanking value:')
blankfield = WIDGET_TEXT(commands, VALUE=blankstring, /EDITABLE, XSIZE=9)
dx_label = WIDGET_LABEL(commands, VALUE=' Pixel size:')
dx = WIDGET_TEXT(commands, VALUE=dxstring, /EDITABLE, XSIZE=5)
dy_label = WIDGET_LABEL(commands, VALUE='x')
dy = WIDGET_TEXT(commands, VALUE=dystring, /EDITABLE, XSIZE=5)
dummy_label = WIDGET_LABEL(commands, VALUE=' ')
update_button = WIDGET_BUTTON(commands, VALUE='Update')

WIDGET_CONTROL, tlb, /REALIZE
WIDGET_CONTROL, draw, GET_VALUE=window_index
info = {parent:father, draw_wid:window_index, blank:blankfield, $
	dx:dx, dy:dy, status:status}
WIDGET_CONTROL, tlb, SET_UVALUE=info
WIDGET_CONTROL, status, SET_VALUE=statline

; Initialize
WSET, window_index
device, bypass_translation=0
loadct, 39
showfeld, plotfeld, pixel=pixel, titles=titles
return, tlb
END
