; ******************************************************************
pro logbeamfit, x, a, f, pder
; ******************************************************************
; Function providing the fit to the Delta variance

common beamwidth, beam
ln10=alog(10.0d0)
; prevent numeric errors
if a[0] gt 100.0 then amp=98.0d0+alog10(a[0]) else amp=a[0]
if a[0] lt -100.0 then amp=-98.0d0+alog10(-a[0])
if a[1] gt 20.0 then alpha=18.7d0+alog10(a[1]) else alpha=a[1]
if a[1] lt -20.0 then alpha=-18.7d0+alog10(-a[1])
if a[2] gt 100.0 then noise=98.0d0+alog10(a[2]) else noise=a[2]
if a[2] lt -100.0 then noise=-98.0d0+alog10(-a[2])

k=1.9041d0-0.7681d0*alpha+0.1189d0*alpha^2
; krb=k*(beam/x)^2 < 1.0
krb=atan(0.5d0*!dpi*k*(beam/x)^2)/(0.5d0*!dpi)

g=10.0d0^amp*(1.0d0-krb)*x^alpha + 10.0d0^noise/x^2
f=alog10(g)

if n_params() ge 4 then begin
  dkda=(-0.7681d0+0.2378d0*alpha)/k
  ; dkda=0.0
  hlp=1.0d0/(g*ln10)
  pder=[[hlp*10.0d0^amp*ln10*(1.0d0-krb)*x^alpha], $
        [hlp*10.0d0^amp*x^alpha*((1.0d0-krb)*alog(x)-dkda*krb)], $
        [hlp*10.0d0^noise*ln10/x^2]]
endif
return
end

; *******************************************************************
pro plotfit, xx, yy, limits=xlimits
; *******************************************************************
; overplot fit result

; Limits
if n_elements(xlimits) ne 2 then xlimits=[min(xx),max(xx)]

indices=where((xx ge xlimits[0]) and (xx le xlimits[1]),count)
if count gt 0 then $
	oplot, xx[indices], yy[indices], color=visiblecolor(0.5)
return
end

; *******************************************************************
function fitfunction, res, beam, linear=linear, fitted=fullfitted, $
	fitrange=fitrange, errwgt=errwgt, sigma=errval, failed=failed
; *******************************************************************

common beamwidth, truebeam

xx=double(res[*,0])
yy=double(res[*,1])
wt=double(res[*,2])

if (beam gt 0.0) and (not keyword_set(linear)) then begin
  linearfit=0
  nfitparm=3
  parmset=[0.0d0,0.0d0,0.0d0]
endif else begin
  linearfit=1
  nfitparm=2
  parmset=[0.0d0,0.0d0]
endelse

; Constrain fit range
if n_elements(fitrange) eq 2 then begin
  indices=where((xx ge fitrange[0]) and (xx le fitrange[1]),count)
  if count lt nfitparm then begin
    print, "Not enough valid data points. No fit computed."
    return, parmset
  endif
  xx=(xx[indices])
  yy=(yy[indices])
  wt=(wt[indices])
endif

if n_elements(errwgt) eq 0 then errwgt=0
minwt=min(wt)
ln10=2.302685d0

if linearfit then begin
 if (minwt gt 0.0) and (errwgt gt 0) then weights=wt/(ln10*yy) 
 parmset=linfit(alog10(xx),alog10(yy),sdev=weights,sigma=errval,/double) 
 parmset=[10.0d0^parmset[0],parmset[1]]
 errval=[ln10*parmset[0]*errval[0],errval[1]]
 fullfitted=(res[*,0]^parmset[1])*parmset[0]
 failed=0
endif else begin
 truebeam=double(beam)
 if (minwt gt 0.0) and (errwgt gt 0) then weights=(ln10*yy/wt)^2 
 ; Two steps - linear fit for good initial guess
 parmset=linfit(alog10(xx),alog10(yy)) 
 guess=[parmset[0],parmset[1],parmset[0]-6.0d0]
 itmax=2000
 fitted=curvefit(xx,alog10(yy),weights,guess,errval, $
	function_name="logbeamfit",itmax=itmax,iter=iter,/double, tol=2.0d-5)
 parmset=[10.0d0^guess[0],guess[1],10.0d0^guess[2]]
 errval=[ln10*parmset[0]*errval[0],errval[1],ln10*parmset[2]*errval[2]]
 logbeamfit, res[*,0], guess, fullfitted
 fullfitted=10.0d0^fullfitted
 if (iter le 2) or (iter ge itmax) then failed=1 else failed=0
endelse

return, parmset
end
