; **************************************************************
pro extractfitparms, widget, beam=beam, dot=dot, err=err
; **************************************************************

WIDGET_CONTROL, widget, GET_UVALUE=info
WIDGET_CONTROL, info.beam, GET_VALUE=bx
IF strnumber(bx[0],beam) eq 0 THEN beam=0.0

WIDGET_CONTROL, info.err, GET_VALUE=nvec
err=max(nvec)

dot=info.dot

return
end

; **************************************************************
pro writefitparms, widget, beam=beam, dot=dot, $
	status=status, onlystatus=onlystatus
; **************************************************************

WIDGET_CONTROL, widget, GET_UVALUE=info
if n_elements(status) ne 0 then WIDGET_CONTROL, info.status, SET_VALUE=status

if not keyword_set(onlystatus) then begin
   if n_elements(beam) ne 0 then  begin
	beamstring=STRTRIM(string(beam, format='(G9.3)'),2)
	WIDGET_CONTROL, info.beam, SET_VALUE=beamstring
   endif
   if n_elements(dot) ne 0 then begin
	info.dot=dot
	WIDGET_CONTROL, widget, SET_UVALUE=info
   endif
endif
return
end

; **************************************************************
pro plotverticalline, x
; **************************************************************
; Plot vertical line to show fit range

oplot, [x,x],[10.0^(!y.crange[0]),10.0^(!y.crange[1])],$
	color=visiblecolor(0.4)
return
end

; **************************************************************
function writedeltavar, widget
; **************************************************************
; Write Delta variance into file

extractwidgetdata, widget, delta=fdata, fit=res, filenames=filenames
file=filenames.resfile
dir=filenames.dir
filter="*.var"
filename=dialog_pickfile(file=file,path=dir,get_path=dir, $
			filter=filter,/write)
rs=size(res)

if (filename ne '') then begin
 filenames.resfile=filename
 filenames.dir=dir
 if rs[1] eq 3 then begin
  comments=["Delta variance", "Fit parameters:", $
  STRCOMPRESS(string(res[0,0],res[0,1], format='(" a0=",G10.4," +/-",G9.3)')), $
  STRCOMPRESS(string(res[1,0],res[1,1], format='(" alpha=",G10.4," +/-",G9.3)')), $
  STRCOMPRESS(string(res[2,0],res[2,1], format='(" a1=",G10.4," +/-",G9.3)'))]
 endif else begin
  comments=["Delta variance", "Fit parameters:", $
  STRCOMPRESS(string(res[0,0],res[0,1], format='(" a0=",G11.5," +/-",G10.4)')), $
  STRCOMPRESS(string(res[1,0],res[1,1], format='(" alpha=",G11.5," +/-",G10.4)'))]
 endelse
 writecol, filename, fdata[*,0], fdata[*,1:*], comments=comments
 writewidgetdata, widget, filenames=filenames
 return, 0
endif else return, 1
end

; **************************************************************
pro deltafitcompute, parent
; **************************************************************
; Event handler for the fit widget of the Delta variance
extractfitparms, parent, beam=bx, err=err
extractwidgetdata, parent, delta=fdata, limits=xlimits
res=fitfunction(fdata,bx,errwgt=err,sigma=errlev,fitrange=xlimits,$
	fitted=fitted,failed=failed)
plotfit, fdata[*,0],fitted,limits=xlimits
fdata=[[fdata[*,0]],[fdata[*,1]],[fdata[*,2]],[fitted]]
res=[[res],[errlev]]
if failed eq 1 then begin
 status='Fit failed to converge! Try another fit range.'
endif else begin
 if bx eq 0.0 then begin
 ; Output for power law fit
  status=STRCOMPRESS(string(res[0,0],res[0,1],res[1,0],res[1,1], $
   format='("a0=",G10.4,"(+/-",G9.4,"), alpha=",G10.4,"(+/-",G9.4,")")'))
 endif else begin
 ; Output for fit with characteristic function
  status=STRCOMPRESS(string(res[0,0],res[0,1],res[1,0],res[1,1],res[2,0],res[2,1], $
   format='(" a0=",G9.3,"(+/-",G8.3,"), alpha=",G9.3,"(+/-",G8.3,"), a1=",G9.3,"(+/-",G8.3,")")'))
 endelse
endelse
writewidgetdata, parent, delta=fdata, fit=res, beam=bx
writefitparms, parent, status=status, /onlystatus

return
end

; **************************************************************
PRO DeltaFitEvent, event
; **************************************************************
; Event handler for the fit widget of the Delta variance

; Switch to active drawing area
WIDGET_CONTROL, event.top, GET_UVALUE=info
wset, info.draw_wid

IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_BUTTON') THEN BEGIN
WIDGET_CONTROL, event.id, GET_VALUE=button_value

; Button handling
CASE button_value OF
'Fit help': XDISPLAYFILE, "fithelp.txt", title="Help on Delta variance fitting"
'Close': WIDGET_CONTROL, event.top, /DESTROY
'Clear': BEGIN
	extractwidgetdata, event.top, delta=fdata
	plotgraph, fdata, titles=['lag','!4r!DD!N!3!U2!N']
	 END
'Lower limit': BEGIN
	extractfitparms, event.top, dot=dot
	extractwidgetdata, event.top, limits=xlimits
	if dot lt xlimits[1] then begin
	  xlimits[0]=dot
	  statline=STRTRIM(string(xlimits, format='("x(min) =",G10.4," x(max) =",G10.4)'),2)
	writewidgetdata, event.top, limits=xlimits
	endif else begin
	  statline='Lower limit must be smaller than upper limit. Limit not set.'
	endelse
	writefitparms, event.top, status=statline, /onlystatus
	       END
'Upper limit': BEGIN
	extractfitparms, event.top, dot=dot
	extractwidgetdata, event.top, limits=xlimits
	if dot gt xlimits[0] then begin
	  xlimits[1]=dot
	  statline=STRTRIM(string(xlimits, format='("x(min) =",G10.4," x(max) =",G10.4)'),2)
	writewidgetdata, event.top, limits=xlimits
	endif else begin
	  statline='Upper limit must be bigger than lower limit. Limit not set.'
	endelse
	writefitparms, event.top, status=statline, /onlystatus
	       END
'Print': BEGIN
	extractwidgetdata, event.top, delta=fdata, limits=xlimits, filenames=filenames
	file=filenames.printfile
	dir=filenames.dir
	filter="*.ps"
	tit="Select Postscript File for Writing"
	filename=dialog_pickfile(file=file,path=dir,get_path=dir, $
				filter=filter,/write,title=tit)
	if (filename ne '') then begin
	  filenames.printfile=filename
	  filenames.dir=dir
	  set_plot, "PS"
	  device, filename=filename, /landscape, /color, bits_per_pixel=24, $
		xsize=22, ysize=15, xoffset=0.5, yoffset=24.5
	  plotgraph, fdata, titles=['lag','!4r!DD!N!3!U2!N']
	  ; Fit plot
	  fsize=size(fdata)
	  if fsize[2] ge 4 then begin
	    plotfit, fdata[*,0],fdata[*,3],limits=xlimits
	  endif
	  device, /close
	  set_plot, "X"
	  writewidgetdata, event.top, filenames=filenames
	  statline="File "+filename+" sucessfully written."
	  writefitparms, event.top, status=statline, /onlystatus
	endif
	 END
'Save': BEGIN
	if writedeltavar(event.top) eq 0 then begin
	  statline="Delta variance file sucessfully written."
	  writefitparms, event.top, status=statline, /onlystatus
	endif
	END
'Compute fit': deltafitcompute, event.top
ENDCASE
ENDIF


; Enter on input fields is equal to Update above
IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_TEXT_CH') THEN BEGIN
 if event.type eq 0 then deltafitcompute, event.top
ENDIF

; Mouse click in the drawing widget - Set limits
IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_DRAW') THEN BEGIN
	xyz=CONVERT_COORD(event.x,event.y,/DEVICE, /TO_DATA)
	; Button pressed
	if event.type eq 0 then begin
	  plotverticalline, xyz[0]
	  statline=STRTRIM(string(xyz[0], format='("lag =",G10.4)'),1)
	  writefitparms, event.top, dot=xyz[0], status=statline
	endif
ENDIF
RETURN
END

; ******************************************************************
function FitWidget, father
; ******************************************************************
; Main program - Definition of the Widget for the Delta variance fit

; ----------------------------------
; First part - data handling
; ----------------------------------
extractwidgetdata, father, delta=res, beam=beam, limits=xlimits
; Input field
rs=size(res)
if (rs[0] ne 2) or (rs[1] lt 3) or (rs[2] lt 2) or (rs[2] gt 4) then begin
	statline="No Delta variance data"
endif else begin
	statline=STRTRIM(string(xlimits, $
		format='("x(min) =",G10.4," , x(max) =",G10.4)'),2)
endelse
; Beam
beamstring=STRTRIM(string(beam, format='(G9.3)'),2)

; ----------------------------------
; Now create the widget
; ----------------------------------
boldfont="-misc-fixed-bold-r-normal--13-120-75-75-c-70-iso8859-1"

tlb= WIDGET_BASE(ROW=3, TITLE="Fitting the Delta variance spectrum", $
		mbar=bar, group_leader=father, event_pro="DeltaFitEvent")

;
; Menu bar
;
file_menu = widget_button(bar, value="File", /menu)
; Not implemented yet
; read_button = WIDGET_BUTTON(file_menu, VALUE='Load')
save_button = WIDGET_BUTTON(file_menu, VALUE='Save')
print_button = WIDGET_BUTTON(file_menu, VALUE='Print')
bye_button = WIDGET_BUTTON(file_menu, VALUE='Close')

help_menu = widget_button(bar, value="Help", /help)
help_btn = widget_button(help_menu, value="Fit help")

;
; Draw area
;
draw = WIDGET_DRAW(tlb, XSIZE=1120, YSIZE=800, COLORS=128, RETAIN=2, /BUTTON_EVENTS)
status = WIDGET_LABEL(tlb, VALUE="Initstring", /DYNAMIC_RESIZE, FONT=boldfont) 

;
; Parameter area
;
commands = WIDGET_BASE(tlb, ROW=1)
first_button = WIDGET_BUTTON(commands, VALUE='Lower limit')
second_button = WIDGET_BUTTON(commands, VALUE='Upper limit')

beam_label = WIDGET_LABEL(commands, VALUE=' Beam size:')
beam = WIDGET_TEXT(commands, VALUE=beamstring, /EDITABLE, XSIZE=6)
err = CW_BGROUP(commands, [""],LABEL_LEFT=" Error weighing:",/NONEXCLUSIVE,SET_VALUE=[1])

fit_button = WIDGET_BUTTON(commands, VALUE='Compute fit')
clear_button = WIDGET_BUTTON(commands, VALUE='Clear')

WIDGET_CONTROL, tlb, /REALIZE
WIDGET_CONTROL, draw, GET_VALUE=window_index
info = {parent:father, draw_wid:window_index, beam:beam, err:err, $
		status:status, dot:xlimits[0]}
WIDGET_CONTROL, tlb, SET_UVALUE=info
WIDGET_CONTROL, status, SET_VALUE=statline

; Initialize
WSET, window_index
device, bypass_translation=0
loadct, 39
plotgraph, res, titles=['lag','!4r!DD!N!3!U2!N']

RETURN, tlb
END
