; ****************************************************************
function noticks, tickstring
; ****************************************************************
; Remove the leading and ending ticks from a Fits string

fin=strpos(tickstring,"'")
fou=strpos(tickstring,"'",/reverse_search)
if fou-fin lt 1 then begin
   fin=-1
   fou=strlen(tickstring)
endif
return, strmid(tickstring,fin+1,fou-fin-1)
end


; ****************************************************************
function headerscl, header, blank=blank, yscale=yscale, zscale=zscale, $
	minmax=minmax, xtitle=xtitle, ytitle=ytitle, ztitle=ztitle, $
	x0=x0, y0=y0, z0=z0
; ****************************************************************
; NAME:
;	HEADERSCL
;
; PURPOSE:
;	Read the axis definitions from a FITs header string
;
; INPUTS:
;	header: 	FITS header string

on_error,2                      ;Return to caller if an error occurs

; Declare types and constants
nstrings=19
fixedstr=["NAXIS", "NAXIS1", "NAXIS2", "NAXIS3",$
	  "CRPIX1", "CRPIX2", "CRPIX3", "CRVAL1", "CRVAL2", "CRVAL3", $
	  "CDELT1", "CDELT2", "CDELT3", "BLANK", "DATAMIN", "DATAMAX", $
	  "CD1_1", "CD2_2", "CD3_3"]
nnames=3
namestr= ["CTYPE1", "CTYPE2", "CTYPE3"]

; Defaults
valarr=double([2.0,128.0,128.0,1.0,1.0,1.0,1.0,0.0,0.0,0.0,1.0,1.0,1.0,$
	-1.0,0.0,0.0,1.0,1.0,1.0])
namearr=make_array(nnames,value=' ')
title=' ' & dummy=' ' & values=' '

; Scan the header
for i=0,n_elements(header)-1 do begin
  reads, header[i], title, dummy, values, format='(A7,A3,A20)'
  for j=0,nstrings-1 do begin
    if strtrim(title) EQ fixedstr[j] then begin
       if strnumber(values,wert) eq 1 then valarr[j]=wert
    endif
  endfor
  for j=0,nnames-1 do begin
    if title EQ namestr[j] then begin
       namearr[j]=strtrim(values,2)
    endif
  endfor
endfor

; Create the vectors
if (valarr[10] eq 1.0d0) and (valarr[16] ne 1.0d0) then $
				dx=valarr[16] else dx=valarr[10]
if (valarr[11] eq 1.0d0) and (valarr[17] ne 1.0d0) then $
				dy=valarr[17] else dy=valarr[11]
if (valarr[12] eq 1.0d0) and (valarr[18] ne 1.0d0) then $
				dz=valarr[18] else dz=valarr[12]
; x axis
nx=fix(valarr[1])
xscale=dblarr(nx)
for i=0,nx-1 do xscale[i]=(i+1-fix(valarr[4])) * dx + valarr[7]
x0=valarr[7]

if fix(valarr[0]) gt 1 then begin
 ny=fix(valarr[2])
 yscale=dblarr(ny)
 for i=0,ny-1 do yscale[i]=(i+1-fix(valarr[5])) * dy + valarr[8]
 y0=valarr[8]
endif

if fix(valarr[0]) gt 2 then begin
 nz=fix(valarr[3])
 zscale=dblarr(nz)
 for i=0,nz-1 do zscale[i]=(i+1-fix(valarr[6])) * dz + valarr[9]
 z0=valarr[9]
endif

blank=valarr[13]
if valarr[15] gt valarr[14] then minmax=[valarr[14],valarr[15]] $
	else minmax=0

; String treatment for the axis names
xtitle=noticks(namearr[0])
ytitle=noticks(namearr[1])
ztitle=noticks(namearr[2])

return, xscale
end
