; **************************************************************
pro extractwgtparms, widget, zero=zero, range=range
; **************************************************************

WIDGET_CONTROL, widget, GET_UVALUE=info

WIDGET_CONTROL, info.zero, GET_VALUE=zw
IF strnumber(zw[0],zero) eq 0 THEN zero=0.0d0

range=[zero,1.0d0]
WIDGET_CONTROL, info.upper, GET_VALUE=uw
IF strnumber(uw[0],upper) eq 0 THEN range[1]=1.0d0 else range[1]=upper
WIDGET_CONTROL, info.lower, GET_VALUE=ul
IF strnumber(ul[0],lower) eq 0 THEN range[0]=zero else range[0]=lower

return
end

; **************************************************************
pro writewgtparms, widget, status=status, onlystatus=onlystatus
; **************************************************************

WIDGET_CONTROL, widget, GET_UVALUE=info
if n_elements(status) ne 0 then WIDGET_CONTROL, info.status, SET_VALUE=status
return
end

; **************************************************************
PRO DeltaWgtEvent, event
; **************************************************************
; Event handler for the weight function widget of the Delta variance

; Switch to active drawing area
WIDGET_CONTROL, event.top, GET_UVALUE=info
wset, info.draw_wid

IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_BUTTON') THEN BEGIN
WIDGET_CONTROL, event.id, GET_VALUE=button_value

; Button handling
CASE button_value OF
'Weighing help': XDISPLAYFILE, "wgthelp.txt", title="Help on weights"
'Close': WIDGET_CONTROL, event.top, /DESTROY
'Normalize': BEGIN
	extractwgtparms, event.top, zero=zero
	extractwidgetdata, event.top, weights=fdata, titles=titles, pixel=pixel
	bas=double(zero)
	fdata=(fdata-bas)/(max(fdata)-bas)
	showfeld, fdata, pixel=pixel, titles=titles
  	statline=STRTRIM(string(min(fdata),max(fdata), $
		format='("f(min) =",G10.4," , f(max) =",G10.4)'),2)
	writewidgetdata, event.top, weights=fdata
	writewgtparms, event.top, status=statline, /onlystatus
	 END
'Update': BEGIN
	extractwgtparms, event.top, zero=zero, range=range
	extractwidgetdata, event.top, weights=fdata, titles=titles, pixel=pixel
	idx=where(fdata lt range[0],count)
	if count gt 0 then fdata[idx]=zero
	idx=where(fdata gt range[1],count)
	if count gt 0 then fdata[idx]=range[1]
	showfeld, fdata, pixel=pixel, titles=titles
  	statline=STRTRIM(string(min(fdata),max(fdata), $
		format='("f(min) =",G10.4," , f(max) =",G10.4)'),2)
	writewidgetdata, event.top, weights=fdata
	writewgtparms, event.top, status=statline, /onlystatus
	 END
'Load': BEGIN
	if loadfield(event.top,/weights) eq 0 then begin
	  extractwidgetdata, event.top, data=fdata, weights=feld, $
		pixel=pixel, titles=titles
	  showfeld, feld, pixel=pixel, titles=titles
  	  statline=STRTRIM(string(min(feld),max(feld), $
		format='("f(min) =",G10.4," , f(max) =",G10.4)'),2)
	  writewgtparms, event.top, status=statline, /onlystatus
	  if comparefieldsizes(feld,fdata) ne 0 then begin
	    bummer, event.top,['Weight field size does not match', $
				'the size of the data field!','',$
				'Analysis impossible.','']
	  endif
	endif
	     END
'Compute from data field': BEGIN
	extractwidgetdata, event.top, data=feld, blank=blank, $
		pixel=pixel, titles=titles
	wgt=simpleweights(feld,blank=blank)
	showfeld, wgt, pixel=pixel, titles=titles
  	  statline=STRTRIM(string(min(wgt),max(wgt), $
	format='("f(min) =",G10.4," , f(max) =",G10.4)'),2)
	writewidgetdata, event.top, weights=wgt
	writewgtparms, event.top, status=statline, /onlystatus
			   END
ENDCASE
ENDIF

; Enter on input fields is equal to Update above
IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_TEXT_CH') THEN BEGIN
 if event.type eq 0 then begin
	extractwgtparms, event.top, zero=zero, range=range
	extractwidgetdata, event.top, weights=fdata, titles=titles, pixel=pixel
	idx=where(fdata lt range[0],count)
	if count gt 0 then fdata[idx]=zero
	idx=where(fdata gt range[1],count)
	if count gt 0 then fdata[idx]=range[1]
	showfeld, fdata, pixel=pixel, titles=titles
  	statline=STRTRIM(string(min(fdata),max(fdata), $
		format='("f(min) =",G10.4," , f(max) =",G10.4)'),2)
	writewidgetdata, event.top, weights=fdata
	writewgtparms, event.top, status=statline, /onlystatus
  endif
ENDIF

; Mouse click in the drawing widget - Examine values
IF (TAG_NAMES(event, /STRUCTURE_NAME) EQ 'WIDGET_DRAW') THEN BEGIN
	; Button pressed
	if event.type eq 0 then begin
	xyz=CONVERT_COORD(event.x,event.y,/DEVICE, /TO_DATA)
	extractwidgetdata, event.top, weights=fdata, titles=titles,pixel=pixel
	 inx=size(fdata)
	 xcoord=fix(xyz[0]/pixel[0]+0.5d0)
	 ycoord=fix(xyz[1]/pixel[1]+0.5d0)
	if ((xcoord ge 0) and (xcoord lt inx[1]) and $
			(ycoord ge 0) and (ycoord lt inx[2])) then begin
	   z=fdata[xcoord,ycoord]
	   statline=STRCOMPRESS(titles[0]+string(xyz[0], format='(" = ",G10.4,", ")')+ $
			titles[1]+string(xyz[1], format='(" = ",G10.4,", ")')+ $
			string(z, format='("I = ",G10.4)'))
	endif else statline="Out of data area"
	writewgtparms, event.top, status=statline, /onlystatus
	endif
ENDIF
RETURN
END

; ******************************************************************
function WgtWidget, father
; ******************************************************************
; Main program - Definition of the Widget for the Delta variance fit

; ----------------------------------
; First part - data handling
; ----------------------------------
extractwidgetdata, father, weights=feld, pixel=pixel, titles=titles

; Input field
rs=size(feld)
if (rs[0] ne 2) then begin
	statline="Empty field"
endif else begin
  	statline=STRTRIM(string(min(feld),max(feld), $
		format='("f(min) =",G10.4," , f(max) =",G10.4)'),2)
endelse
; Zero cut
zero=0.0d0
zerostring=STRTRIM(string(zero, format='(G9.3)'),2)
; Upper cut
upper=max(feld)
upperstring=STRTRIM(string(upper, format='(G9.3)'),2)
; Lower cut
lower=zero
lowerstring=STRTRIM(string(lower, format='(G9.3)'),2)
; Pixel size
dxstring=STRTRIM(string(pixel[0], format='(G9.3)'),2)
dystring=STRTRIM(string(pixel[1], format='(G9.3)'),2)

; ----------------------------------
; Now create the widget
; ----------------------------------
boldfont="-misc-fixed-bold-r-normal--13-120-75-75-c-70-iso8859-1"

tlb= WIDGET_BASE(ROW=3, TITLE="Field of weights",mbar=bar, $
		group_leader=father, event_pro="DeltaWgtEvent")

;
; Menu bar
;
file_menu = widget_button(bar, value="File", /menu)
read_button = WIDGET_BUTTON(file_menu, VALUE='Load')
bye_button = WIDGET_BUTTON(file_menu, VALUE='Close')

compute_menu = widget_button(bar, value="Compute", /menu)
deltacompute = widget_button(compute_menu, value="Compute from data field")

help_menu = widget_button(bar, value="Help", /help)
help_btn = widget_button(help_menu, value="Data file help")

;
; Draw area
;
draw = WIDGET_DRAW(tlb, XSIZE=860, YSIZE=760, COLORS=128, RETAIN=2, /BUTTON_EVENTS)
status = WIDGET_LABEL(tlb, VALUE="Initstring", /DYNAMIC_RESIZE, FONT=boldfont) 

;
; Parameter area
;
commands = WIDGET_BASE(tlb, ROW=1)
zero_label = WIDGET_LABEL(commands, VALUE='Zero:')
zerofield = WIDGET_TEXT(commands, VALUE=zerostring, /EDITABLE, XSIZE=5)
range_label = WIDGET_LABEL(commands, VALUE=' Valid range:')
lowerfield = WIDGET_TEXT(commands, VALUE=lowerstring, /EDITABLE, XSIZE=5)
range_dash = WIDGET_LABEL(commands, VALUE='-')
upperfield = WIDGET_TEXT(commands, VALUE=upperstring, /EDITABLE, XSIZE=5)
dummy_label = WIDGET_LABEL(commands, VALUE='  ')
update_button = WIDGET_BUTTON(commands, VALUE='Update')
normal_button = WIDGET_BUTTON(commands, VALUE='Normalize')

WIDGET_CONTROL, tlb, /REALIZE
WIDGET_CONTROL, draw, GET_VALUE=window_index
info = {parent:father, draw_wid:window_index, zero:zerofield, $
	upper:upperfield, lower:lowerfield, status:status}
WIDGET_CONTROL, tlb, SET_UVALUE=info
WIDGET_CONTROL, status, SET_VALUE=statline

; Initialize
WSET, window_index
device, bypass_translation=0
loadct, 39
showfeld, feld, pixel=pixel, titles=titles

RETURN, tlb
END
